"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.commandHandler = void 0;
const configure_1 = require("./commands/configure");
const create_configuration_1 = require("./commands/create-configuration");
const create_organization_1 = require("./commands/create-organization");
const get_resources_1 = require("./commands/get-resources");
const init_1 = require("./commands/init");
const login_1 = require("./commands/login");
const logout_1 = require("./commands/logout");
const publish_1 = require("./commands/publish");
const upload_1 = require("./commands/upload");
const configuration_1 = __importDefault(require("./configuration"));
const prompts_1 = require("./prompts/prompts");
const utils_1 = require("./utils");
async function commandHandler(options) {
    if (!(await (0, utils_1.fileExists)(configuration_1.default.configFilePath))) {
        await (0, create_configuration_1.createConfiguration)();
    }
    if (options._[0] === "login") {
        const validOptions = ["github", "gitlab", "bitbucket"];
        const unknownOptions = Object.keys(options).filter((option) => option !== "_" && option !== "$0" && !validOptions.includes(option));
        if (unknownOptions.length > 0) {
            console.log(`Unrecognized options: ${unknownOptions.join(", ")}`);
            process.exit(1);
        }
        (async () => {
            if (options.github) {
                await (0, login_1.login)("github");
            }
            else if (options.gitlab) {
                await (0, login_1.login)("gitlab");
            }
            else if (options.bitbucket) {
                await (0, login_1.login)("bitbucket");
            }
            else {
                const { provider } = await (0, prompts_1.promptForLogin)();
                await (0, login_1.login)(provider.toLowerCase());
            }
        })();
    }
    if (options._[0] === "logout") {
        (async () => {
            await (0, logout_1.logout)();
        })();
    }
    if (options._[0] === "upload") {
        const validOptions = ["path", "protocol", "project", "organization"];
        const unknownOptions = Object.keys(options).filter((option) => option !== "_" && option !== "$0" && !validOptions.includes(option));
        if (unknownOptions.length > 0) {
            console.log(`Unrecognized options: ${unknownOptions.join(", ")}`);
            process.exit(1);
        }
        (async () => {
            let path, protocol, organizationId, projectName;
            if (options.path && options.protocol) {
                path = options.path;
                protocol = options.protocol.toLowerCase();
                organizationId = options.organization;
                projectName = options.project;
            }
            else {
                const prompt = await (0, prompts_1.promptForUploadFile)();
                path = prompt.path;
                protocol = prompt.protocol.toLowerCase();
                organizationId = prompt.organizationId;
                projectName = prompt.project;
            }
            if (!projectName) {
                const pathSegments = process.cwd().split("/");
                projectName = pathSegments[pathSegments.length - 1];
                console.log(`Generated default project name: ${projectName}`);
            }
            if (!organizationId) {
                organizationId = await (0, utils_1.readFromJsonFile)("organization", configuration_1.default.configFilePath);
            }
            if (!path) {
                path = "./";
            }
            try {
                await (0, upload_1.upload)(path, protocol, organizationId, projectName);
            }
            catch (error) {
                process.exit(1);
            }
        })();
    }
    if (options._[0] === "publish") {
        (async () => {
            try {
                if (options.organization) {
                    await (0, publish_1.publish)(options.organization);
                }
                else {
                    await (0, publish_1.publish)();
                }
            }
            catch (error) {
                process.exit(1);
            }
        })();
    }
    if (options._[0] === "create-organization") {
        const validOptions = ["name", "username"];
        const unknownOptions = Object.keys(options).filter((option) => option !== "_" && option !== "$0" && !validOptions.includes(option));
        if (unknownOptions.length > 0) {
            console.log(`Unrecognized options: ${unknownOptions.join(", ")}`);
            process.exit(1);
        }
        (async () => {
            try {
                let name, username;
                if (options.name && options.username) {
                    name = options.name;
                    username = options.username;
                }
                else {
                    const prompt = await (0, prompts_1.promptForCreateOrganization)();
                    name = prompt.name;
                    username = prompt.username;
                }
                if (!name) {
                    throw new Error("Please insert a name for organization.");
                }
                if (!username) {
                    throw new Error("Please insert username for organization.");
                }
                try {
                    await (0, create_organization_1.createOrganization)(name, username, "app");
                }
                catch (error) {
                    process.exit(1);
                }
            }
            catch (error) {
                console.log(error.message);
                process.exit(1);
            }
        })();
    }
    if (options._[0] === "init") {
        const validOptions = ["protocol", "project", "path", "framework"];
        const unknownOptions = Object.keys(options).filter((option) => option !== "_" && option !== "$0" && !validOptions.includes(option));
        if (unknownOptions.length > 0) {
            console.log(`Unrecognized options: ${unknownOptions.join(", ")}`);
            process.exit(1);
        }
        (async () => {
            try {
                let project, protocol, path, framework;
                if (options.protocol) {
                    project = options.project;
                    protocol = options.protocol.toLowerCase();
                    path = options.path;
                    framework = options.framework;
                }
                else {
                    const prompt = await (0, prompts_1.promptForInit)();
                    project = prompt.project;
                    protocol = prompt.protocol.toLowerCase();
                    path = prompt.path;
                    framework = prompt.framework.toLowerCase();
                }
                if (!project) {
                    const pathSegments = process.cwd().split("/");
                    project = pathSegments[pathSegments.length - 1];
                }
                if (!path) {
                    path = "./";
                }
                await (0, init_1.init)(project, protocol, path, framework);
            }
            catch (error) {
                console.log(error.message);
                process.exit(1);
            }
        })();
    }
    if (options._[0] === "create-dapp") {
        (async () => {
            try {
                if (options._[1]) {
                    await (0, prompts_1.promptForCreateDapp)(options._[1]);
                }
                else {
                    await (0, prompts_1.promptForCreateDapp)();
                }
            }
            catch (error) {
                console.log(error.message);
                process.exit(1);
            }
        })();
    }
    if (options._[0] === "get") {
        (async () => {
            try {
                if (options.resource) {
                    if (options.resource == get_resources_1.ResourceEnum.DEPLOYMENT) {
                        const id = options.id;
                        await get_resources_1.ResourceFetcher.getDeployment(id);
                    }
                    else if (options.resource == get_resources_1.ResourceEnum.DEPLOYMENTS) {
                        const projectId = options.projectId;
                        const skip = options.skip;
                        const limit = options.limit;
                        const status = options.status;
                        await get_resources_1.ResourceFetcher.getProjectDeployments(projectId, skip, limit, status);
                    }
                    else if (options.resource == get_resources_1.ResourceEnum.PROJECT) {
                        const id = options.id;
                        await get_resources_1.ResourceFetcher.getProject(id);
                    }
                    else if (options.resource == get_resources_1.ResourceEnum.PROJECTS) {
                        const organizationId = options.organizationId;
                        const skip = options.skip;
                        const limit = options.limit;
                        const state = options.state;
                        await get_resources_1.ResourceFetcher.getOrganizationProjects(organizationId, skip, limit, state);
                    }
                    else if (options.resource == get_resources_1.ResourceEnum.ORGANIZATION) {
                        const id = options.id;
                        await get_resources_1.ResourceFetcher.getOrganization(id);
                    }
                    else if (options.resource == get_resources_1.ResourceEnum.ORGANIZATIONS) {
                        await get_resources_1.ResourceFetcher.getUserOrganizations();
                    }
                    else if (options.resource == get_resources_1.ResourceEnum.DOMAINS) {
                        const projectId = options.projectId;
                        await get_resources_1.ResourceFetcher.getProjectDomains(projectId);
                    }
                    else if (options.resource == get_resources_1.ResourceEnum.DEPLOYMENT_ENVIRONMENTS) {
                        const projectId = options.projectId;
                        await get_resources_1.ResourceFetcher.getProjectDeploymentEnvironments(projectId);
                    }
                }
                else {
                    throw new Error("Resource needs to be specified");
                }
            }
            catch (error) {
                console.log(error.message);
                process.exit(1);
            }
        })();
    }
    if (options._[0] === "configure") {
        const validOptions = ["organization"];
        const unknownOptions = Object.keys(options).filter((option) => option !== "_" && option !== "$0" && !validOptions.includes(option));
        if (unknownOptions.length > 0) {
            console.log(`Unrecognized options: ${unknownOptions.join(", ")}`);
            process.exit(1);
        }
        (async () => {
            try {
                let organizationId;
                if (options.organization) {
                    organizationId = options.organization;
                }
                else {
                    const prompt = await (0, prompts_1.promptForConfigure)();
                    organizationId = prompt.organization;
                }
                await (0, configure_1.changeDefaultOrganization)(organizationId);
            }
            catch (error) {
                console.log(error.message);
                process.exit(1);
            }
        })();
    }
    if (!options._[0]) {
        console.log("Please use --help to check all commands available with spheron cli");
    }
}
exports.commandHandler = commandHandler;
